﻿//
//	eyemLib・ヘッダー
//
#ifndef __EYEM_LIB_H
#define __EYEM_LIB_H

#include <float.h>

#define MAC_OSX

/********************************************************************************************/
/*                                     共通ヘッダー                                         */
/********************************************************************************************/

// 一般定義
#define FUNC_OK						0						// 正常終了
#define FUNC_NOT_ENOUGH_MEM			(-1)					// ワークメモリ不足
#define FUNC_ILLEGAL_ARGUMENT		(-2)					// 引数が不適当
#define FUNC_CANNOT_CALC			(-100)					// 計算不可
#define FUNC_CANNOT_USE				(-999)					// 使用不可

// エラーコード（行列計算）
#define FUNC_DET_EQ_ZERO			(-110)					// 行列式がゼロ
#define FUNC_FAILED_EIGEN			(-111)					// 固有値・固有ベクトル計算失敗
#define FUNC_FAILED_SVD				(-112)					// 特異値分解計算失敗
#define FUNC_FAILED_CHOLESKY		(-113)					// コレスキー分解計算失敗

// エラーコード（近似計算）
#define FUNC_FAILED_ROBUST			(-130)					// ロバスト推定失敗
#define FUNC_FAILED_ELLIPSE			(-131)					// 楕円にならなかった
#define FUNC_FAILED_ELLIPSOID		(-132)					// 楕円体にならなかった
#define FUNC_FAILED_CONE			(-133)					// 錐面にならなかった

// エラーコード（カメラキャリブレーション）
#define FUNC_FAILED_HOMOGRAPHY		(-150)					// ホモグラフィー行列計算失敗
#define FUNC_FAILED_CAM_PRM			(-151)					// カメラパラメータ計算失敗
#define FUNC_FAILED_BUNDLE_ADJ		(-152)					// バンドル調整失敗
#define FUNC_FAILED_UNDISTORT		(-153)					// 歪み補正失敗


// 定数
#define EPS				1.0e-9								// ε
#define DBL_EPS			DBL_EPSILON							// ε(機械イプシロン)
#define PI				3.1415926535897932384626433832795	// π
#define PI_DEG			180.0								// π(deg)
#define RAD2DEG			(PI_DEG/PI)							// rad→deg変換
#define DEG2RAD			(PI/PI_DEG)							// deg→rad変換
#define TWO_PI			(2.0*PI)							// 2π
#define PI_BY_2			(PI/2.0)							// π/2


// 比較
#ifndef MIN
#define MIN( a, b ) ( ((a) < (b)) ? (a) : (b) )
#endif
#ifndef MAX
#define MAX( a, b ) ( ((a) > (b)) ? (a) : (b) )
#endif


#define ON				1
#define OFF				0

#ifndef TRUE
#define TRUE			1
#endif

#ifndef FALSE
#define FALSE			0
#endif

#ifndef NULL
#ifdef __cplusplus
	#define NULL		0
#else
	#define NULL		((void *)0)
#endif
#endif


// 画像境界処理
#ifndef __EYEM_BORDER
#define __EYEM_BORDER
enum{
	EYEM_BORDER_NONE,										// 境界処理はしない
	EYEM_BORDER_REPLICATE,									// 最外周濃度値を延長
	EYEM_BORDER_REFLECT,									// 鏡映（最外周に対して対称にコピー）
	EYEM_BORDER_ZERO										// 濃度ゼロで埋める
};
#endif


// 画像情報
typedef struct {
	unsigned char			*ucpImage;						// 先頭アドレス
	int						iWidth;							// 画像メモリｘ方向サイズ
	int						iHeight;						// 画像メモリｙ方向サイズ
}	EyemImage;


#ifndef __APPLE__
typedef unsigned long vImagePixelCount;
typedef struct vImage_Buffer { 
	void *data; 
	vImagePixelCount height;
	vImagePixelCount width;
	size_t rowBytes;
} vImage_Buffer;
#endif


// 矩形定義
typedef struct {
	int						iXs;							// 始点(左上)ｘ座標
	int						iYs;							// 始点(左上)ｙ座標
	int						iWidth;							// ｘ方向サイズ（幅）
	int						iHeight;						// ｙ方向サイズ（高さ）
}	EyemRect;

typedef struct {
	int						iXs;							// 始点(左上)ｘ座標
	int						iYs;							// 始点(左上)ｙ座標
	int						iXe;							// 終点(右下)ｘ座標
	int						iYe;							// 終点(右下)ｙ座標
}	EyemRect2;


///////////////////////////////////////////////////////////////////////////////
// Define point or parameter on Orthogonal Coordinate System

/////////////////////
// int type
//
typedef struct{
	int						iX;								// Ｘ座標
	int						iY;								// Ｙ座標
}	EyemOcsIXY;

typedef struct{
	int						iX;								// Ｘ座標
	int						iY;								// Ｙ座標
	int						iZ;								// Ｚ座標
}	EyemOcsIXYZ;

typedef struct{
	int						iX;								// Ｘ座標
	int						iY;								// Ｙ座標
	int						iQ;								// θ
}	EyemOcsIXYQ;

typedef struct{
	int						iX;								// Ｘ座標
	int						iY;								// Ｙ座標
	int						iR;								// 半径
}	EyemOcsIXYR;											// 円の表現に使用

typedef struct{
	int						iA;								// a
	int						iB;								// b
	int						iC;								// c
}	EyemOcsIABC;											// 直線(一般形)の表現に使用

typedef struct{
	int						iR;								// ρ
	int						iQ;								// θ
}	EyemOcsIRQ;												// 直線(ヘッセの標準形)やベクトルの表現に使用

typedef struct{
	int						iX;								// Ｘ座標(単位：画素)
	int						iY;								// Ｙ座標(単位：画素)
	int						iQ;								// 傾き(単位：rad)
	int						iS;								// スケール
}	EyemOcsIXYQS;


/////////////////////
// float type
//
typedef struct{
	float					fX;								// Ｘ座標
	float					fY;								// Ｙ座標
}	EyemOcsFXY;

typedef struct{
	float					fX;								// Ｘ座標
	float					fY;								// Ｙ座標
	float					fZ;								// Ｚ座標
}	EyemOcsFXYZ;

typedef struct{
	float					fX;								// Ｘ座標
	float					fY;								// Ｙ座標
	float					fQ;								// θ
}	EyemOcsFXYQ;

typedef struct{
	float					fX;								// Ｘ座標
	float					fY;								// Ｙ座標
	float					fR;								// 半径
}	EyemOcsFXYR;											// 円の表現に使用

typedef struct{
	float					fA;								// a
	float					fB;								// b
	float					fC;								// c
}	EyemOcsFABC;											// 直線(一般形)の表現に使用

typedef struct{
	float					fR;								// ρ
	float					fQ;								// θ
}	EyemOcsFRQ;												// 直線(ヘッセの標準形)やベクトルの表現に使用

typedef struct{
	float					fX;								// Ｘ座標(単位：画素)
	float					fY;								// Ｙ座標(単位：画素)
	float					fQ;								// 傾き(単位：rad)
	float					fS;								// スケール
}	EyemOcsFXYQS;


/////////////////////
// double type
//
typedef struct{
	double					dX;								// Ｘ座標
	double					dY;								// Ｙ座標
}	EyemOcsDXY;

typedef struct{
	double					dX;								// Ｘ座標
	double					dY;								// Ｙ座標
	double					dZ;								// Ｚ座標
}	EyemOcsDXYZ;

typedef struct{
	double					dX;								// Ｘ座標
	double					dY;								// Ｙ座標
	double					dQ;								// θ
}	EyemOcsDXYQ;

typedef struct{
	double					dX;								// 中心のＸ座標
	double					dY;								// 中心のＹ座標
	double					dR;								// 半径
}	EyemOcsDXYR;											// 円の表現に使用

typedef struct{
	double					dA;								// a
	double					dB;								// b
	double					dC;								// c
}	EyemOcsDABC;											// 直線(一般形)の表現に使用

typedef struct{
	double					dR;								// ρ
	double					dQ;								// θ
}	EyemOcsDRQ;												// 直線(ヘッセの標準形)やベクトルの表現に使用

typedef struct{
	double					dX;								// Ｘ座標
	double					dY;								// Ｙ座標
	double					dQ;								// 回転角(単位：rad)
	double					dS;								// スケール
}	EyemOcsDXYQS;

typedef struct{
	double					dA;								// a
	double					dB;								// b
	double					dC;								// c
	double					dD;								// d
}	EyemOcsDABCD;											// 平面(一般形)の表現に使用

typedef struct{
	double					dXo;							// 中心のＸ座標
	double					dYo;							// 中心のＹ座標
	double					dL;								// 長軸半径
	double					dS;								// 短軸半径
	double					dQ;								// 長軸の傾き(単位：rad)
}	EyemOcsDXYLSQ;											// 楕円の表現に使用

typedef struct{
	EyemOcsDXYZ				tP;								// 直線上の１点の座標
	EyemOcsDXYZ				tV;								// 直線の方向ベクトル
}	EyemOcsDPV;												// ３次元空間内の直線の表現に使用


typedef struct{
	EyemOcsDXYZ				tC;								// 楕円体の中心
	EyemOcsDXYZ				tR;								// 軸の半径(dX：長軸, dY：中軸, dZ：短軸)
	double					dU;								// 長軸のＸＹ平面への射影がＸ軸となす角(単位：rad)
	double					dV;								// 長軸のＸＹ平面とのなす角(単位：rad)
	double					dW;								// 長軸まわりの回転角(単位：rad)
}	EyemOcsDCRUVW;											// 楕円体の表現に使用


/********************************************************************************************/
/*                             以下，各ソース固有のヘッダー                                 */
/********************************************************************************************/

//////////////////////////////////////////////////////////////////////////////////////////////
// 数学計算ユーティリティ（eyemMath.c）
//
#ifdef __cplusplus
extern "C" {
#endif

double	eyemMathCalcInnerProduct( int, double [], double [] );
void	eyemMathCalcOuterProduct( double [], double [], double [] );
int		eyemMathCalcAngle( int, double [], double [], double * );
double	eyemMathCalcNorm( int, double [] );
double	eyemMathCalcArgument( double [] );
int		eyemMathNormalization( int, double [] );
int		eyemMathStat( int, double [], double *, double *, double * );
double	eyemMathMedianI( int, int * );
double	eyemMathMedianD( int, double * );
int		eyemMathOtsuThreshold1d( int, double [], double * );
double	eyemMathAreaTriangle( double, double, double, double, double, double );
double	eyemMathSignAreaTriangle( double, double, double, double, double, double );
void	eyemMathRotatePoint( double, double, double, double, double, double *, double * );
void	eyemMathCreateTransParam( int, double, double, double, double, double *, double *, double * );
void	eyemMathTransCoordOfPoint( double, double, double, double, double, double *, double * );
void	eyemMathInvTransCoordOfPoint( double, double, double, double, double, double *, double * );
void	eyemMathComposeCoord( double, double, double, double, double, double, double *, double *, double * );
double	eyemMathInvMatrixOfSyn3X3( double [][3], double [][3] );
void	eyemMathInvCoord( double, double, double, double *, double *, double * );
double  eyemMathGetDistFromPointToPoint( double, double, double, double );
double	eyemMathGetDistFromPointToLine( double, double, double, double, double );
double	eyemMathGetDistAndCrossPointFromPointToLine( double, double, double, double, double, double *, double * );
double	eyemMathGetDistFromPointToCircle( double, double, double, double, double, double *, double * );
int		eyemMathGetDistFromPointToEllipse( double, double, double, double, double, double, double, double *, double *, double * );
int		eyemMathCrossPoint( double, double, double, double, double, double, double *, double * );
void	eyemMathTransAbcToRq( double, double, double, double *, double * );
double	eyemMathCrossAngle( double, double, double, double, double, double );
int		eyemMathGetLineFrom2Points( double, double, double, double, double *, double *, double * );
void	eyemMathGetOrthogonalLineFromLineAndPoint( double, double, double, double, double *, double *, double * );
int		eyemMathCheckAngle( double, double, double );
double	eyemMathAddAngle( int, int, double, double );
double	eyemMathPrimeAngle( int, int, double );
double	eyemMathExtremumOfQuadraticCurves(  double, double, double, double * );
double	eyemMathExtremumOfQuadraticSurface(  double, double, double, double, double, double, double *, double * );
double	eyemMathCrossCorrelation( int, double [], double [] );
int		eyemMathAutoCorrelation( int, double [], int, double [] );
double	eyemMathNormCorrelation( int, double [], double [] );
int		eyemMathNormAutoCorrelation( int, double [], int, double [] );
int		eyemMathSearchOf1DWithNormCorrelation( int, double [], int, double [], int *, double * );
int		eyemMathQuadraticRoots( double [], double [] );
int		eyemMathCubicRoots( double [], double [] );
int		eyemMathQuarticRoots( double [], double [] );
double	eyemMathHorner( int, double	[], double );
void	eyemMathTransCoord( double, double, double, double, double, double *, double * );

#ifdef __cplusplus
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////
// 行列計算ユーティリティ（eyemMat.c）
//
#ifdef __cplusplus
extern "C" {
#endif

// プロトタイプ
double	*eyemMatMallocVector( int );
double	**eyemMatMallocMatrix( int, int );
void	eyemMatFreeVector( double * );
void	eyemMatFreeMatrix( double ** );
void	eyemMatZero( int, int, void * );
void	eyemMatDiagonal( double, int, void * );
void	eyemMatCopy( int, int, void *, void	* );
void	eyemMatAdd( int, int, void *, void *, void * );
void	eyemMatSub( int, int, void *, void *, void * );
void	eyemMatMulScalar( double, int, int, void *, void * );
int		eyemMatMul( int, int, void *, int, int, void *, void * );
int		eyemMatGenMul( double, int, int, void *, int, int, void	*, double, void	*, void	* );
void	eyemMatMulTransposed( int, int, int, void	*, void	* );
int		eyemMatMulTransposed2( int, int, int, void *, int, int, void *, void * );
void	eyemMatTranspose( int, int, void *, void * );
double	eyemMatTrace( int, void	* );
int		eyemMatInvert( int, void *, void * );
int		eyemMatSolveLU( int, void *, double [], double	[] );
int		eyemMatSolveSVD( int, int, void	*, double [], double [] );
int		eyemMatJacobiEigen( int, void *, double [], void * );
int		eyemMatQrEigen( int, void *, double [], void * );
int		eyemMatSVD( int, int, void *, void *, double [], void * );
int		eyemMatCholeskyDecomp( int, void *, void * );

#ifdef __cplusplus
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////
// 近似計算（ロバスト推定）（eyemFit.c）
//
#ifdef __cplusplus
extern "C" {
#endif

// プロトタイプ
int		eyemFitMean1dim( int, double [], int, double, double * );
int		eyemFitMean2dim( int, EyemOcsDXY [], int, double, EyemOcsDXY * );
int		eyemFitMean3dim( int, EyemOcsDXYZ [], int, double, EyemOcsDXYZ * );
int		eyemFitLine( int, EyemOcsDXY [], int, double, EyemOcsDABC * );
int		eyemFitLineIn3d( int, EyemOcsDXYZ [], int, double, EyemOcsDPV * );
int		eyemFitPlane( int, EyemOcsDXYZ [], int, double, EyemOcsDABCD * );
int		eyemFitCircle( int, EyemOcsDXY [], int, double, EyemOcsDXYR * );
int		eyemFitEllipse( int, EyemOcsDXY [], int, double, EyemOcsDXYLSQ * );
int		eyemFitEllipseC( int , EyemOcsDXY [], int, double, double [] );
int		eyemFitConics( int, EyemOcsDXY [], int, double, double [] );
int		eyemFitParabola( int, EyemOcsDXY [], int, double, EyemOcsDABC * );
int		eyemFitEllipsoid( int, EyemOcsDXYZ [], int, double, EyemOcsDCRUVW * );
int		eyemFitCone( int, EyemOcsDXYZ [], int, double, double [] );

#ifdef __cplusplus
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////
// ２次元幾何計算（２次元キャリパー）（eyemClp2d.c）
//
#ifdef __cplusplus
extern "C" {
#endif

// プロトタイプ
void	eyemClp2dDistanceTwoPoints( EyemOcsDXY *, EyemOcsDXY *, double * );
void	eyemClp2dCenterTwoPoints( EyemOcsDXY *, EyemOcsDXY *, EyemOcsDXY * );
int		eyemClp2dLineTwoPoints( EyemOcsDXY *, EyemOcsDXY *, EyemOcsDABC * );
int		eyemClp2dMidperpendicularTwoPoints( EyemOcsDXY *, EyemOcsDXY *, EyemOcsDABC * );
int		eyemClp2dVerticalLinePointAndLine( EyemOcsDXY *, EyemOcsDABC *, EyemOcsDABC * );
void	eyemClp2dLinePointAndSlope( EyemOcsDXY *, double, EyemOcsDABC * );
int		eyemClp2dIntersectionTwoLines( EyemOcsDABC *, EyemOcsDABC *, EyemOcsDXY * );
int		eyemClp2dAngleTwoLines( EyemOcsDABC *, EyemOcsDABC *, double * );
int		eyemClp2dCenterLineOfTwoLines( EyemOcsDABC *, EyemOcsDABC *, EyemOcsDABC * );
int		eyemClp2dDistancePointToLine( EyemOcsDXY *, EyemOcsDABC *, double * );
int		eyemClp2dFootOfPerpendicularToLine( EyemOcsDXY *, EyemOcsDABC *, EyemOcsDXY * );
int		eyemClp2dTranslationOfLine( EyemOcsDABC *, EyemOcsDXY *, EyemOcsDABC * );
void	eyemClp2dAreaTriangle( EyemOcsDXY *, EyemOcsDXY *, EyemOcsDXY *, double * );
void	eyemClp2dSignAreaTriangle( EyemOcsDXY *, EyemOcsDXY *, EyemOcsDXY *, double * );
int		eyemClp2dCircleThreePoints( EyemOcsDXY *, EyemOcsDXY *, EyemOcsDXY *, EyemOcsDXYR * );
void	eyemClp2dCircleTwoPoints( EyemOcsDXY *, EyemOcsDXY *, EyemOcsDXYR * );
int		eyemClp2dIntersectionLineAndCircle( EyemOcsDABC *, EyemOcsDXYR *, EyemOcsDXY *, EyemOcsDXY * );
int		eyemClp2dTangentPointToCircle( EyemOcsDXY *, EyemOcsDXYR *, EyemOcsDABC *, EyemOcsDXY *, EyemOcsDABC *, EyemOcsDXY * );
int		eyemClp2dClosestToCircle( EyemOcsDXY *, EyemOcsDXYR *, EyemOcsDXY *, double * );

#ifdef __cplusplus
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////
// ３次元幾何計算（３次元キャリパー）（eyemClp3d.c）
//
#ifdef __cplusplus
extern "C" {
#endif

// プロトタイプ
void	eyemClp3dDistanceTwoPoints( EyemOcsDXYZ *, EyemOcsDXYZ *, double * );
void	eyemClp3dCenterTwoPoints( EyemOcsDXYZ *, EyemOcsDXYZ *, EyemOcsDXYZ	* );
int		eyemClp3dLineTwoPoints( EyemOcsDXYZ	*, EyemOcsDXYZ *, EyemOcsDPV * );
int		eyemClp3dFootOfPerpendicularToLine( EyemOcsDXYZ *, EyemOcsDPV	*, EyemOcsDXYZ * );
int		eyemClp3dVerticalLinePointAndLine( EyemOcsDXYZ	*, EyemOcsDPV *, EyemOcsDPV	* );
int		eyemClp3dDistancePointToLine( EyemOcsDXYZ *, EyemOcsDPV *, double * );
int		eyemClp3dDistanceTwoLines( EyemOcsDPV *, EyemOcsDPV *, double * );
int		eyemClp3dPlaneThreePoints( EyemOcsDXYZ *, EyemOcsDXYZ *, EyemOcsDXYZ *, EyemOcsDABCD * );
int		eyemClp3dVerticalPlanePointAndVector( EyemOcsDXYZ *, EyemOcsDXYZ *, EyemOcsDABCD * );
int		eyemClp3dVerticalLinePointAndPlane( EyemOcsDXYZ *, EyemOcsDABCD	*, EyemOcsDPV * );
int		eyemClp3dDistancePointToPlane( EyemOcsDXYZ *, EyemOcsDABCD *, double * );
int		eyemClp3dIntersectionLineAndPlane( EyemOcsDPV *, EyemOcsDABCD *, EyemOcsDXYZ * );
int		eyemClp3dAngleLineAndPlane( EyemOcsDPV *, EyemOcsDABCD *, double * );
int		eyemClp3dFootOfPerpendicularToPlane( EyemOcsDXYZ *, EyemOcsDABCD *, EyemOcsDXYZ * );
int		eyemClp3dIntersectionTwoPlanes( EyemOcsDABCD	*, EyemOcsDABCD	*, EyemOcsDPV * );
int		eyemClp3dAngleTwoPlanes( EyemOcsDABCD *, EyemOcsDABCD *, double * );

#ifdef __cplusplus
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////
// 計算幾何（eyemCg.c）
//

// 線分交差状態
enum {
	EYEM_CG_NOT_INTERSECTION,					// 交差していない
	EYEM_CG_INTERSECTION,						// 交差している（互いに他をまたぐ）
	EYEM_CG_END1_CONTACT_END2,					// 線分１の端点と線分２の端点が接触（一致）
	EYEM_CG_END1_CONTACT_SGM2,					// 線分１の端点が線分２の端点以外で接触
	EYEM_CG_END2_CONTACT_SGM1,					// 線分２の端点が線分１の端点以外で接触
	EYEM_CG_SEG1_OVERLAP_SGM2					// 線分１と線分２が重なる（２線分が同一直線上にある場合）
};


// 点位置状態
enum {
	EYEM_CG_OUTER,								// 外部にある
	EYEM_CG_INNER,								// 内部にある
	EYEM_CG_BORDER								// 境界上にある
};


#ifdef __cplusplus
extern "C" {
#endif

// プロトタイプ
int		eyemCgSortByArgument( int n, EyemOcsDXY taSrcPt[], EyemOcsDXY taDstPt[] );
int		eyemCgConvexHull( int, EyemOcsDXY [], int *, EyemOcsDXY [] );
int		eyemCgSmallestEnclosingCircle( int, EyemOcsDXY [], EyemOcsDXY *, double * );
int		eyemCgIntersectionOfTwoSegments( EyemOcsDXY *, EyemOcsDXY *, EyemOcsDXY *, EyemOcsDXY *, int *, EyemOcsDXY * );
int		eyemCgPointInConvexPolygon( int, EyemOcsDXY [], EyemOcsDXY *, int * );
int		eyemCgIntersectionLineAndConvexPolygon( int, EyemOcsDXY [], EyemOcsDABC *, EyemOcsDXY [] );
int		eyemCgIntersectionSegmentAndConvexPolygon( int, EyemOcsDXY [], EyemOcsDXY *, EyemOcsDXY *, EyemOcsDXY [] );
int		eyemCgPointInPolygon( int, EyemOcsDXY [], EyemOcsDXY *, int * );
int		eyemCgAreaOfPolygon( int, EyemOcsDXY [], double * );

#ifdef __cplusplus
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////
// ２値ブロッブ解析（eyemBin.c）
//

// ２値ブロッブ：色
enum {
	EYEM_BIN_BLACK,								// 黒
	EYEM_BIN_WHITE								// 白
};


// ブロッブの解析結果
typedef struct {
	int				iArea;						// 面積
	double			dCenterX;					// 重心ｘ座標
	double			dCenterY;					// 重心ｙ座標
	int				iXs, iYs, iXe, iYe;			// 外接長方形（始点，終点）
	int				iWidth, iHeight;			// 外接長方形（ｘ方向サイズ(幅)，ｙ方向サイズ(高さ)）
	double			dTheta;						// 主軸傾斜角（rad）
}	EyemBinBlob;


#ifdef __cplusplus
extern "C" {
#endif

// プロトタイプ
int		eyemBinOtsuThresholdImage( EyemImage *, EyemRect * );
int		eyemBinOtsuThresholdGray( int, double [] );
int		eyemBinOtsuThresholdHistogram( int, unsigned int [] );
int		eyemBinBinaryImage( EyemImage *, EyemRect *, int, EyemImage * );
int		eyemBinDilation( EyemImage *, EyemRect *, int, int, EyemImage * );
int		eyemBinErosion( EyemImage *, EyemRect *, int, int, EyemImage * );
int		eyemBinOpening( EyemImage *, EyemRect *, int, int, EyemImage * );
int		eyemBinClosing( EyemImage *, EyemRect *, int, int, EyemImage * );
int		eyemBinBlob( EyemImage *, EyemRect *, int, int, int, int, EyemBinBlob **, int * );
void	eyemBinFree( EyemBinBlob ** );
int		eyemBinSauvola( EyemImage *, EyemRect *, int, int, double, double, int, EyemImage * );

#ifdef __cplusplus
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////
// １次元エッジ抽出（eyemEdge1d.c）
//
#ifdef __cplusplus
extern "C" {
#endif

// プロトタイプ
int		eyemEdge1dDiffMax( EyemImage *, EyemOcsDXY *, EyemOcsDXY *, int, EyemOcsDXY *, double * );
int		eyemEdge1dGenDiffMax( EyemImage *, EyemOcsDXY *, EyemOcsDXY *, int, EyemOcsDXY *, double * );
int		eyemEdge1dGenNearest( EyemImage *, EyemOcsDXY *, EyemOcsDXY *, int, EyemOcsDXY *, double * );
int		eyemEdge1dGenNearestC( EyemImage *, EyemOcsDXY *, EyemOcsDXY *, int, EyemOcsDXY *, double * );
int		eyemEdge1dGenGradMax( EyemImage *, EyemOcsDXY *, EyemOcsDXY *, int, EyemOcsDXY *, double * );
int		eyemEdge1dManual( EyemImage *, EyemOcsDXY *, EyemOcsDXY *, int, int [], EyemOcsDXY *, double * );
int		eyemEdge1dManualG( EyemImage *, EyemOcsDXY *, EyemOcsDXY *, int, double [], int, int [], EyemOcsDXY *, double * );

#ifdef __cplusplus
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////
// 点群マッチング（eyemMatch.c）
//

// 処理種別
enum {
	EYEM_MATCH_LSQ,			// 最小二乗法・スケール推定なし
	EYEM_MATCH_LSQ_S,		// 最小二乗法・スケール推定あり
	EYEM_MATCH_ROBUST,		// ロバスト推定・スケール推定なし
	EYEM_MATCH_ROBUST_S,	// ロバスト推定・スケール推定あり
	EYEM_MATCH_MINMAX,		// ミニマックス法・スケール推定なし
	EYEM_MATCH_MINMAX_S		// ミニマックス法・スケール推定あり
};


#ifdef __cplusplus
extern "C" {
#endif

// プロトタイプ
int		eyemMatchBasic( int, EyemOcsDXY [], EyemOcsDXY [], EyemOcsDXYQ *, int, double, EyemOcsDXYQS * );
int		eyemMatchDelta( EyemOcsDXYQ *, EyemOcsDXYQS *, double *, double *, double *, double * );
int		eyemMatchPoints( int, EyemOcsDXY [], EyemOcsDXY [], EyemOcsDXYQ *, EyemOcsDXYQS *, EyemOcsDXY [], double [] );

#ifdef __cplusplus
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////
// カメラキャリブレーション（eyemCalib.c）
//

// パターンデータ
typedef struct {
	int				iPtnRefN;					// パターン参照点の個数
	EyemOcsDXY		*tpObjPt;					// パターン参照点の実座標（単位：mm）
	EyemOcsDXY		*tpImgPt;					// パターン参照点の画像座標（単位：画素）
}	EyemCalibPtn;


// ホモグラフィー行列
typedef struct {
	double			daH[3][3];					// ホモグラフィー行列H
	double			daInvH[3][3];				// 行列Hの逆行列
}	EyemCalibHom;


// カメラの内部パラメータ（intrinsic parameter）
typedef struct {
	double			daA[3][3];					// 内部パラメータ行列
	double			dFx, dFy;					// 焦点距離×スケール（単位：画素）
	double			dFs;						// 焦点距離×せん断係数（単位：画素）
	double			dUo, dVo;					// 画像中心座標（単位：画素）
	double			dTheta;						// （未使用）画像面傾斜角（単位：rad）
	double			dK1, dK2;					// ラジアル歪み係数
	double			dK3, dK4, dK5;				// ラジアル歪み係数
	double			dP1, dP2;					// 偏心歪み係数
	double			daInvA[3][3];				// 内部パラメータ行列の逆行列
	double			daRsv[5];					// 予備領域
}	EyemCalibInt;


// カメラの外部パラメータ（extrinsic parameter）
typedef struct {
	double			daR[3][3];					// 回転行列
	double			daT[3];						// 並進ベクトル
	double			daRdr[3];					// 回転ベクトル（ロドリゲスの表現）
}	EyemCalibExt;


#ifdef __cplusplus
extern "C" {
#endif

// プロトタイプ
int		eyemCalibCalcHomography( int, void *, void *, double, void * );
int		eyemCalibCameraParam( int, EyemCalibPtn [], double, EyemCalibInt *, EyemCalibExt [], double *, int );
int		eyemCalibRodrigues( int, double [3], double [3][3] );
int		eyemCalibNormalizeData( int, int, void *, void *, void *, void * );
int		eyemCalibUnDistortPoint( EyemCalibInt *, double, double, double *, double * );
int		eyemCalibDistortPoint( EyemCalibInt *, double, double, double *, double * );
int		eyemCalibCameraMatrix( EyemCalibInt *, EyemCalibExt *, double [3][4], int );
int		eyemCalibChange3Dto2D( EyemOcsDXYZ *, EyemCalibInt *, EyemCalibExt *, EyemOcsDXY *, int );

#ifdef __cplusplus
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////
// 画像平滑化フィルタ（eyemSmooth.c）
//
#ifdef __cplusplus
extern "C" {
#endif

// プロトタイプ
int		eyemSmoothGaussian( EyemImage *, EyemRect *, int, int, EyemImage * );
int		eyemSmoothMedian( EyemImage *, EyemRect *, int, int, EyemImage * );

#ifdef __cplusplus
}
#endif


//////////////////////////////////////////////////////////////////////////////////////////////
// その他のツール（eyemMisc.c）
//
#ifdef __cplusplus
extern "C" {
#endif

// プロトタイプ
int		eyemMiscChangeRectToRect2( EyemRect *, EyemRect2 * );
int		eyemMiscChangeRect2ToRect( EyemRect2 *, EyemRect * );
double	eyemMiscBilinearGrayLevel( EyemImage *, double, double );
int		eyemMiscPlotPointsOfLine( EyemOcsIXY *, EyemOcsIXY *, EyemOcsIXY [] );
int		eyemMiscExpandBorder( EyemImage *, EyemRect *, int, int, int, EyemImage *, EyemRect *, EyemRect * );

#ifdef __cplusplus
}
#endif


#endif/* __EYEM_LIB_H */
